import { DisposableLike } from "@reactive-js/core/lib/disposable";
import { Function1 } from "@reactive-js/core/lib/functions";
import { ObservableLike } from "@reactive-js/core/lib/observable";
import { HttpContentEncoding, HttpRequest, HttpResponse } from "./interfaces";
export declare const enum HttpClientRequestStatusType {
    Start = 1,
    Progress = 2,
    Completed = 3,
    HeadersReceived = 4
}
export declare type HttpClientRequestStatusStart = {
    readonly type: HttpClientRequestStatusType.Start;
};
export declare type HttpClientRequestStatusProgress = {
    readonly type: HttpClientRequestStatusType.Progress;
    readonly count: number;
};
export declare type HttpClientRequestStatusComplete = {
    readonly type: HttpClientRequestStatusType.Completed;
};
export declare type HttpClientRequestStatusHeadersReceived<TResp extends DisposableLike> = {
    readonly type: HttpClientRequestStatusType.HeadersReceived;
    readonly response: HttpResponse<TResp>;
};
export declare type HttpClientRequestStatus<TResp extends DisposableLike> = HttpClientRequestStatusStart | HttpClientRequestStatusProgress | HttpClientRequestStatusComplete | HttpClientRequestStatusHeadersReceived<TResp>;
export declare type HttpClientRequest<T> = HttpRequest<T> & {
    readonly acceptedEncodings?: readonly HttpContentEncoding[];
    readonly maxRedirects?: number;
};
export declare type HttpClient<THttpRequest extends HttpClientRequest<unknown>, TResp extends DisposableLike> = (req: THttpRequest) => ObservableLike<HttpClientRequestStatus<TResp>>;
export declare const withDefaultBehaviors: <TReq, TResp extends DisposableLike>(encodeHttpRequest?: Function1<HttpClientRequest<TReq>, HttpClientRequest<TReq>>) => (httpClient: HttpClient<HttpClientRequest<TReq>, TResp>) => HttpClient<HttpClientRequest<TReq>, TResp>;
