import { Function1, SideEffect2 } from "@reactive-js/core/lib/functions";
import { IOSourceLike } from "@reactive-js/core/lib/io";
import { HttpStatusCode, HttpHeaders, URILike, HttpResponse, HttpRequest, EntityTag, CacheDirective, HttpContentEncoding, MediaRange, MediaType } from "./interfaces";
export declare const createHttpResponse: <T>({ body, cacheControl, contentInfo, etag, expires, headers, lastModified, location, preferences, statusCode, vary, ...rest }: {
    body: T;
    cacheControl?: readonly (string | CacheDirective)[] | undefined;
    contentInfo?: {
        contentEncodings?: readonly HttpContentEncoding[] | undefined;
        contentLength?: number | undefined;
        contentType: string | MediaType;
    } | undefined;
    etag?: string | EntityTag | undefined;
    expires?: number | undefined;
    headers?: import("@reactive-js/core/lib/option").Option<HttpHeaders>;
    lastModified?: string | number | Date | undefined;
    location?: string | URILike | undefined;
    preferences?: {
        acceptedCharsets?: import("@reactive-js/core/lib/option").Option<readonly string[]>;
        acceptedEncodings?: readonly HttpContentEncoding[] | undefined;
        acceptedLanguages?: import("@reactive-js/core/lib/option").Option<readonly string[]>;
        acceptedMediaRanges?: readonly (string | MediaRange)[] | undefined;
    } | undefined;
    statusCode: HttpStatusCode;
    vary?: import("@reactive-js/core/lib/option").Option<readonly string[]>;
}) => HttpResponse<T>;
export declare const parseHttpResponseFromHeaders: <T>(statusCode: number, headers: HttpHeaders, body: T) => HttpResponse<T>;
export declare const writeHttpResponseHeaders: <T>(response: HttpResponse<T>, writeHeader: SideEffect2<string, string>) => void;
export declare const checkIfNotModified: <T>({ cacheControl, method, preconditions, }: HttpRequest<unknown>) => Function1<HttpResponse<T>, HttpResponse<T>>;
export declare const encodeHttpResponseWithUtf8: Function1<HttpResponse<string>, HttpResponse<Uint8Array>>;
export declare const decodeHttpResponseWithCharset: Function1<HttpResponse<Uint8Array>, HttpResponse<string>>;
export declare const toIOSourceHttpResponse: <TBody>(resp: HttpResponse<TBody>) => HttpResponse<IOSourceLike<TBody>>;
export declare const decodeHttpResponseContent: (decoderProvider: {
    [key: string]: Function1<IOSourceLike<Uint8Array>, IOSourceLike<Uint8Array>>;
}) => Function1<HttpResponse<IOSourceLike<Uint8Array>>, HttpResponse<IOSourceLike<Uint8Array>>>;
export declare const encodeHttpResponseContent: (encoderProvider: {
    [key: string]: Function1<IOSourceLike<Uint8Array>, IOSourceLike<Uint8Array>>;
}, db?: {
    [key: string]: {
        compressible?: boolean | undefined;
    };
}) => (request: HttpRequest<unknown>) => Function1<HttpResponse<IOSourceLike<Uint8Array>>, HttpResponse<IOSourceLike<Uint8Array>>>;
export declare const createHttpErrorResponse: (e: unknown) => HttpResponse<unknown>;
