"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const disposable_1 = require("@reactive-js/core/lib/disposable");
const functions_1 = require("@reactive-js/core/lib/functions");
const observable_1 = require("@reactive-js/core/lib/observable");
const option_1 = require("@reactive-js/core/lib/option");
const http_1 = require("../../http");
const capabilities_1 = require("./capabilities");
const httpResponseBody_1 = require("./httpResponseBody");
const loadBodyContent = (response) => __awaiter(void 0, void 0, void 0, function* () {
    const { body, contentInfo } = response;
    if (option_1.isSome(contentInfo)) {
        const { contentLength, contentType: { type, subtype, params }, } = contentInfo;
        const hasCharset = option_1.isSome(params["charset"]);
        const responseIsText = hasCharset ||
            type === "text" ||
            subtype.includes("json") ||
            subtype.includes("text") ||
            subtype.includes("xml");
        if (responseIsText) {
            return body.text();
        }
        else if (contentLength > 0 && capabilities_1.supportsArrayBuffer) {
            return body.arrayBuffer();
        }
        else if (capabilities_1.supportsBlob) {
            return body.blob();
        }
    }
    return yield "";
});
exports.sendHttpRequestUsingFetch = request => {
    const { cache, credentials, integrity, method, mode, redirect, referrerPolicy, uri, } = request;
    const url = uri.toString();
    const headers = http_1.httpRequestToUntypedHeaders(request);
    const fetchResponse = observable_1.createObservable((dispatcher) => __awaiter(void 0, void 0, void 0, function* () {
        const abortController = new AbortController();
        disposable_1.addTeardown(dispatcher, () => abortController.abort());
        try {
            const fetchResponse = yield fetch(url, {
                cache,
                credentials,
                headers,
                integrity,
                method,
                mode,
                redirect,
                referrerPolicy,
                signal: abortController.signal,
            });
            const responseHeaders = {};
            fetchResponse.headers.forEach((v, k) => {
                responseHeaders[k] = v;
            });
            const response = http_1.parseHttpResponseFromHeaders(fetchResponse.status, responseHeaders, fetchResponse);
            observable_1.dispatch(dispatcher, response);
            disposable_1.dispose(dispatcher);
        }
        catch (cause) {
            disposable_1.dispose(dispatcher, { cause });
        }
    }));
    const mapResponseBody = observable_1.switchMap((response) => observable_1.using(scheduler => functions_1.pipe(functions_1.defer(response, loadBodyContent), observable_1.fromPromise, observable_1.publish(scheduler, 1), body => new httpResponseBody_1.HttpResponseBodyImpl(body)), body => observable_1.fromValue()(Object.assign(Object.assign({}, response), { body }))));
    return observable_1.concat(observable_1.fromValue()({
        type: 1,
    }), functions_1.pipe(fetchResponse, mapResponseBody, observable_1.map(response => ({
        type: 4,
        response,
    }))));
};
