"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const disposable_1 = require("@reactive-js/core/lib/disposable");
const functions_1 = require("@reactive-js/core/lib/functions");
const observable_1 = require("@reactive-js/core/lib/observable");
const blobToString = (blob) => {
    const onSubscribe = (dispatcher) => {
        const reader = new FileReader();
        reader.onload = functions_1.defer(reader.result, observable_1.dispatchTo(dispatcher));
        reader.onerror = () => disposable_1.dispose(dispatcher, { cause: reader.error });
        reader.readAsText(blob);
    };
    return observable_1.createObservable(onSubscribe);
};
const blobToArrayBuffer = (body) => {
    const onSubscribe = (dispatcher) => {
        const reader = new FileReader();
        reader.onload = functions_1.defer(reader.result, observable_1.dispatchTo(dispatcher));
        reader.onerror = () => disposable_1.dispose(dispatcher, { cause: reader.error });
        reader.readAsArrayBuffer(body);
    };
    return observable_1.createObservable(onSubscribe);
};
const throwTypeError = () => {
    throw new TypeError("invalid type");
};
const bodyToArrayBuffer = (body) => {
    return typeof body === "string" || body instanceof Blob
        ? blobToArrayBuffer(new Blob([body]))
        : body instanceof ArrayBuffer
            ? observable_1.fromValue()(body)
            : throwTypeError();
};
const bodyToBlob = (body) => {
    return typeof body === "string" || body instanceof ArrayBuffer
        ? observable_1.fromValue()(new Blob([body]))
        : body instanceof Blob
            ? observable_1.fromValue()(body)
            : throwTypeError();
};
const bodyToText = (body) => {
    return typeof body === "string"
        ? observable_1.fromValue()(body)
        : body instanceof Blob
            ? blobToString(body)
            : body instanceof ArrayBuffer
                ? blobToString(new Blob([body]))
                : throwTypeError();
};
class HttpResponseBodyImpl extends disposable_1.AbstractDisposable {
    constructor(body) {
        super();
        this.body = body;
        disposable_1.bindDisposables(this, body);
    }
    get arrayBuffer() {
        return functions_1.pipe(this.body, observable_1.await_(bodyToArrayBuffer));
    }
    get blob() {
        return functions_1.pipe(this.body, observable_1.await_(bodyToBlob));
    }
    get text() {
        return functions_1.pipe(this.body, observable_1.await_(bodyToText));
    }
}
exports.HttpResponseBodyImpl = HttpResponseBodyImpl;
