"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const disposable_1 = require("@reactive-js/core/lib/disposable");
const functions_1 = require("@reactive-js/core/lib/functions");
const observable_1 = require("@reactive-js/core/lib/observable");
const option_1 = require("@reactive-js/core/lib/option");
const http_1 = require("../../http");
const capabilities_1 = require("./capabilities");
const httpResponseBody_1 = require("./httpResponseBody");
exports.sendHttpRequestUsingXHR = request => observable_1.createObservable(dispatcher => {
    const xhr = new XMLHttpRequest();
    const xhrSupportsResponseType = "responseType" in xhr;
    const bodyStream = observable_1.createSubject(1);
    const body = new httpResponseBody_1.HttpResponseBodyImpl(bodyStream);
    disposable_1.bindDisposables(body, dispatcher);
    disposable_1.addTeardown(dispatcher, () => xhr.abort());
    xhr.onerror = () => {
        const cause = new Error("Network request failed");
        disposable_1.dispose(dispatcher, { cause });
    };
    xhr.onreadystatechange = () => {
        var _a;
        if (xhr.readyState === 2) {
            const headersRaw = (_a = xhr.getAllResponseHeaders()) !== null && _a !== void 0 ? _a : "";
            const headers = http_1.parseHeaders(headersRaw);
            const response = http_1.parseHttpResponseFromHeaders(xhr.status, headers, body);
            observable_1.dispatch(dispatcher, {
                type: 4,
                response,
            });
            const { contentInfo } = response;
            if (xhrSupportsResponseType && option_1.isSome(contentInfo)) {
                const { contentLength, contentType: { type, subtype, params }, } = contentInfo;
                const hasCharset = option_1.isSome(params["charset"]);
                const responseIsText = hasCharset ||
                    type === "text" ||
                    subtype.includes("json") ||
                    subtype.includes("text") ||
                    subtype.includes("xml");
                xhr.responseType = responseIsText
                    ? "text"
                    : contentLength > 0 && capabilities_1.supportsArrayBuffer
                        ? "arraybuffer"
                        : capabilities_1.supportsBlob
                            ? "blob"
                            : "";
            }
        }
        else if (xhr.readyState === 4) {
            observable_1.dispatch(bodyStream, xhr.response);
        }
    };
    xhr.onloadstart = functions_1.defer({
        type: 1,
    }, observable_1.dispatchTo(dispatcher));
    xhr.upload.onprogress = ev => {
        const { loaded: count } = ev;
        observable_1.dispatch(dispatcher, {
            type: 2,
            count,
        });
    };
    xhr.upload.onload = functions_1.defer({
        type: 3,
    }, observable_1.dispatchTo(dispatcher));
    xhr.ontimeout = () => {
        const cause = new Error("Network request failed");
        disposable_1.dispose(dispatcher, { cause });
    };
    if (request.credentials === "include") {
        xhr.withCredentials = true;
    }
    else if (request.credentials === "omit") {
        xhr.withCredentials = false;
    }
    xhr.open(request.method, request.uri.toString(), true);
    http_1.writeHttpRequestHeaders(request, (k, v) => xhr.setRequestHeader(k, v));
    xhr.send(request.body);
});
