import { dispose, bindDisposables, addTeardown, } from "@reactive-js/core/lib/disposable.mjs";
import { defer } from "@reactive-js/core/lib/functions.mjs";
import { createObservable, createSubject, dispatch, dispatchTo, } from "@reactive-js/core/lib/observable.mjs";
import { isSome } from "@reactive-js/core/lib/option.mjs";
import { parseHeaders, parseHttpResponseFromHeaders, writeHttpRequestHeaders, } from "../../http.mjs";
import { supportsArrayBuffer, supportsBlob } from "./capabilities.mjs";
import { HttpResponseBodyImpl } from "./httpResponseBody.mjs";
export const sendHttpRequestUsingXHR = request => createObservable(dispatcher => {
    const xhr = new XMLHttpRequest();
    const xhrSupportsResponseType = "responseType" in xhr;
    const bodyStream = createSubject(1);
    const body = new HttpResponseBodyImpl(bodyStream);
    bindDisposables(body, dispatcher);
    addTeardown(dispatcher, () => xhr.abort());
    xhr.onerror = () => {
        const cause = new Error("Network request failed");
        dispose(dispatcher, { cause });
    };
    xhr.onreadystatechange = () => {
        var _a;
        if (xhr.readyState === 2) {
            const headersRaw = (_a = xhr.getAllResponseHeaders()) !== null && _a !== void 0 ? _a : "";
            const headers = parseHeaders(headersRaw);
            const response = parseHttpResponseFromHeaders(xhr.status, headers, body);
            dispatch(dispatcher, {
                type: 4,
                response,
            });
            const { contentInfo } = response;
            if (xhrSupportsResponseType && isSome(contentInfo)) {
                const { contentLength, contentType: { type, subtype, params }, } = contentInfo;
                const hasCharset = isSome(params["charset"]);
                const responseIsText = hasCharset ||
                    type === "text" ||
                    subtype.includes("json") ||
                    subtype.includes("text") ||
                    subtype.includes("xml");
                xhr.responseType = responseIsText
                    ? "text"
                    : contentLength > 0 && supportsArrayBuffer
                        ? "arraybuffer"
                        : supportsBlob
                            ? "blob"
                            : "";
            }
        }
        else if (xhr.readyState === 4) {
            dispatch(bodyStream, xhr.response);
        }
    };
    xhr.onloadstart = defer({
        type: 1,
    }, dispatchTo(dispatcher));
    xhr.upload.onprogress = ev => {
        const { loaded: count } = ev;
        dispatch(dispatcher, {
            type: 2,
            count,
        });
    };
    xhr.upload.onload = defer({
        type: 3,
    }, dispatchTo(dispatcher));
    xhr.ontimeout = () => {
        const cause = new Error("Network request failed");
        dispose(dispatcher, { cause });
    };
    if (request.credentials === "include") {
        xhr.withCredentials = true;
    }
    else if (request.credentials === "omit") {
        xhr.withCredentials = false;
    }
    xhr.open(request.method, request.uri.toString(), true);
    writeHttpRequestHeaders(request, (k, v) => xhr.setRequestHeader(k, v));
    xhr.send(request.body);
});
