"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const functions_1 = require("@reactive-js/core/lib/functions");
const parserCombinators_1 = require("@reactive-js/core/lib/internal/parserCombinators");
const option_1 = require("@reactive-js/core/lib/option");
const readonlyArray_1 = require("@reactive-js/core/lib/readonlyArray");
const httpGrammar_1 = require("./httpGrammar");
const httpHeaders_1 = require("./httpHeaders");
const pOptionalEquals = parserCombinators_1.optional(parserCombinators_1.pEquals);
const pCacheDirective = (charStream) => {
    const directive = httpGrammar_1.pToken(charStream);
    const hasValue = option_1.isSome(pOptionalEquals(charStream));
    const value = hasValue ? httpGrammar_1.pTokenOrQuotedString(charStream) : "";
    return { directive, value };
};
exports.parseCacheDirective = parserCombinators_1.parseWith(pCacheDirective);
exports.parseCacheDirectiveOrThrow = parserCombinators_1.parseWithOrThrow(pCacheDirective);
exports.cacheDirectiveToString = ({ directive, value, }) => value.length > 0
    ? `${directive}=${httpGrammar_1.toTokenOrQuotedString(value)}`
    : `${directive}`;
const parseCacheDirectiveList = functions_1.pipe(pCacheDirective, httpGrammar_1.httpList, parserCombinators_1.parseWith);
exports.parseCacheControlFromHeaders = (headers) => {
    var _a;
    const cacheControl = httpHeaders_1.getHeaderValue(headers, "Cache-Control");
    return option_1.isSome(cacheControl)
        ? (_a = parseCacheDirectiveList(cacheControl)) !== null && _a !== void 0 ? _a : [] : [];
};
exports.writeHttpCacheControlHeader = (cacheControl, writeHeader) => {
    if (cacheControl.length > 0) {
        writeHeader("Cache-Control", functions_1.pipe(cacheControl, readonlyArray_1.map(exports.cacheDirectiveToString), readonlyArray_1.join(",")));
    }
};
