"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const disposable_1 = require("@reactive-js/core/lib/disposable");
const functions_1 = require("@reactive-js/core/lib/functions");
const observable_1 = require("@reactive-js/core/lib/observable");
const option_1 = require("@reactive-js/core/lib/option");
const httpRequest_1 = require("./httpRequest");
const redirectCodes = [
    301,
    302,
    303,
    307,
    308,
];
exports.withDefaultBehaviors = (encodeHttpRequest = functions_1.identity) => (httpClient) => {
    const sendRequest = (request) => functions_1.pipe(request, observable_1.fromValue(), observable_1.map(encodeHttpRequest), observable_1.switchMap(httpClient), observable_1.concatMap(status => {
        var _a, _b;
        if (status.type === 4) {
            const { response } = status;
            const { location, preferences, statusCode } = response;
            const acceptedEncodings = (_a = preferences === null || preferences === void 0 ? void 0 : preferences.acceptedEncodings) !== null && _a !== void 0 ? _a : [];
            const shouldRedirect = redirectCodes.includes(statusCode) &&
                option_1.isSome(location) &&
                ((_b = request === null || request === void 0 ? void 0 : request.maxRedirects) !== null && _b !== void 0 ? _b : 10) > 0;
            const newRequest = shouldRedirect
                ? httpRequest_1.createRedirectHttpRequest(request, response)
                : statusCode === 417
                    ? Object.assign(Object.assign({}, request), { expectContinue: false }) : statusCode === 415 &&
                    acceptedEncodings.length > 0
                    ? Object.assign(Object.assign({}, request), { acceptedEncodings }) : request;
            if (request !== newRequest) {
                disposable_1.dispose(response.body);
                return sendRequest(newRequest);
            }
        }
        return observable_1.fromValue()(status);
    }));
    return sendRequest;
};
