"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const functions_1 = require("@reactive-js/core/lib/functions");
const parserCombinators_1 = require("@reactive-js/core/lib/internal/parserCombinators");
const option_1 = require("@reactive-js/core/lib/option");
const readonlyArray_1 = require("@reactive-js/core/lib/readonlyArray");
const httpGrammar_1 = require("./httpGrammar");
const httpHeaders_1 = require("./httpHeaders");
const mediaType_1 = require("./mediaType");
const parseTokenList = functions_1.pipe(httpGrammar_1.pToken, httpGrammar_1.httpList, parserCombinators_1.parseWith);
exports.parseHttpContentInfoFromHeaders = (headers) => {
    var _a, _b, _c;
    const contentEncodingString = (_a = httpHeaders_1.getHeaderValue(headers, "Content-Encoding")) !== null && _a !== void 0 ? _a : "";
    const contentEncodings = parseTokenList(contentEncodingString);
    const contentLengthHeader = (_b = httpHeaders_1.getHeaderValue(headers, "Content-Length")) !== null && _b !== void 0 ? _b : "-1";
    const contentLength = ~~contentLengthHeader;
    const contentType = mediaType_1.parseMediaType((_c = httpHeaders_1.getHeaderValue(headers, "Content-Type")) !== null && _c !== void 0 ? _c : "");
    return option_1.isNone(contentType)
        ? option_1.none
        : {
            contentEncodings,
            contentLength,
            contentType,
        };
};
exports.writeHttpContentInfoHeaders = (content, writeHeader) => {
    const { contentLength, contentType, contentEncodings } = content;
    if (contentLength > 0) {
        writeHeader("Content-Length", contentLength.toString(10));
    }
    writeHeader("Content-Type", mediaType_1.mediaTypeToString(contentType));
    if (contentEncodings.length > 0) {
        writeHeader("Content-Encoding", functions_1.pipe(contentEncodings, readonlyArray_1.join(", ")));
    }
};
exports.createHttpContentInfo = ({ contentEncodings, contentLength, contentType, }) => ({
    contentEncodings: contentEncodings !== null && contentEncodings !== void 0 ? contentEncodings : [],
    contentLength: contentLength !== null && contentLength !== void 0 ? contentLength : -1,
    contentType: typeof contentType === "string"
        ? mediaType_1.parseMediaTypeOrThrow(contentType)
        : contentType,
});
exports.contentIsCompressible = (content, db) => content.contentEncodings.length === 0 &&
    mediaType_1.mediaTypeIsCompressible(content.contentType, db);
