"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const functions_1 = require("@reactive-js/core/lib/functions");
const parserCombinators_1 = require("@reactive-js/core/lib/internal/parserCombinators");
const option_1 = require("@reactive-js/core/lib/option");
const pTChar = parserCombinators_1.satisfy(c => c === 33 ||
    c === 35 ||
    c === 36 ||
    c === 37 ||
    c === 38 ||
    c === 39 ||
    c === 42 ||
    c === 43 ||
    c === 45 ||
    c === 46 ||
    c === 94 ||
    c === 95 ||
    c === 96 ||
    c === 124 ||
    c === 126 ||
    (c >= 48 && c <= 57) ||
    (c >= 97 && c <= 122) ||
    (c >= 65 && c <= 90));
const pWS = parserCombinators_1.satisfy(c => c === 32 || c === 9);
const pOWS = parserCombinators_1.manyIgnore()(pWS);
const pQuotedString = charStream => {
    let builder = option_1.none;
    charStream.move();
    const initialIndex = charStream.index;
    if (charStream.current !== 34) {
        parserCombinators_1.throwParseError(charStream);
    }
    while (charStream.move()) {
        const c = charStream.current;
        const isQDText = c === 9 ||
            c === 32 ||
            c === 33 ||
            (c >= 0x23 && c <= 0x5b) ||
            (c >= 0x5d && c <= 0x7e) ||
            (c >= 0x80 && c <= 0xff);
        if (c === 34) {
            break;
        }
        else if (isQDText) {
            const refinableBuilder = builder;
            if (option_1.isSome(refinableBuilder)) {
                refinableBuilder.push(c);
            }
        }
        else if (c === 92 && charStream.move()) {
            if (option_1.isNone(builder)) {
                builder = [];
            }
            const c = charStream.current;
            const isQuotedPairChar = c === 9 ||
                c === 32 ||
                (c >= 0x21 && c <= 0x7e) ||
                (c >= 0x80 && c <= 0xff);
            if (!isQuotedPairChar) {
                parserCombinators_1.throwParseError(charStream);
            }
            builder.push(c);
        }
        else {
            parserCombinators_1.throwParseError(charStream);
        }
    }
    return option_1.isSome(builder)
        ? String.fromCharCode(...builder)
        : charStream.src.substring(initialIndex + 1, charStream.index);
};
exports.pToken = functions_1.pipe(pTChar, parserCombinators_1.manySatisfy({ min: 1 }));
exports.pTokenOrQuotedString = functions_1.pipe(exports.pToken, parserCombinators_1.or(pQuotedString));
const pParameter = (charStream) => {
    const key = exports.pToken(charStream);
    parserCombinators_1.pEquals(charStream);
    const value = exports.pTokenOrQuotedString(charStream);
    return [key, value];
};
exports.toTokenOrQuotedString = (input) => {
    let buffer = option_1.none;
    for (let i = 0; i < input.length; i++) {
        const c = input.charCodeAt(i);
        const isQuotedPairChar = c === 9 ||
            c === 32 ||
            (c >= 0x21 && c <= 0x7e) ||
            (c >= 0x80 && c <= 0xff);
        const isQDText = c === 9 ||
            c === 32 ||
            c === 33 ||
            (c >= 0x23 && c <= 0x5b) ||
            (c >= 0x5d && c <= 0x7e) ||
            (c >= 0x80 && c <= 0xff);
        if (isQuotedPairChar && !isQDText) {
            if (option_1.isNone(buffer)) {
                buffer = [34];
                for (let j = 0; j < i; j++) {
                    const c = input.charCodeAt(j);
                    buffer.push(c);
                }
            }
            buffer.push(92);
        }
        else if (!isQDText) {
            throw new Error();
        }
        if (option_1.isSome(buffer)) {
            buffer.push(c);
        }
    }
    if (option_1.isSome(buffer)) {
        buffer.push(34);
        return String.fromCharCode(...buffer);
    }
    else {
        return input;
    }
};
const pParamsParam = (charStream) => {
    pOWS(charStream);
    parserCombinators_1.pSemicolon(charStream);
    pOWS(charStream);
    return pParameter(charStream);
};
exports.pParams = functions_1.pipe(pParamsParam, parserCombinators_1.many(), parserCombinators_1.map(results => {
    const params = {};
    for (const [k, v] of results) {
        params[k] = v;
    }
    return params;
}));
const owsCommaOws = (charStream) => {
    pOWS(charStream);
    parserCombinators_1.pComma(charStream);
    pOWS(charStream);
};
exports.httpList = (parser) => functions_1.pipe(parser, parserCombinators_1.sepBy(owsCommaOws));
const pFieldVchar = parserCombinators_1.satisfy(c => (c >= 0x21 && c <= 0x7e) || (c >= 0x80 && c <= 0xff));
const pFieldVCharSpHTab = parserCombinators_1.satisfy(c => c === 32 ||
    c === 9 ||
    (c >= 0x21 && c <= 0x7e) ||
    (c >= 0x80 && c <= 0xff));
const parseManyFieldVCharSpHTab = parserCombinators_1.manyIgnore()(pFieldVCharSpHTab);
const pFieldValue = (charStream) => {
    const index = charStream.index + 1;
    pFieldVchar(charStream);
    parseManyFieldVCharSpHTab(charStream);
    charStream.index--;
    pFieldVchar(charStream);
    return charStream.src.substring(index, charStream.index + 1);
};
const pCRLF = parserCombinators_1.string("\r\n");
const pHeaders = (charStream) => {
    const result = {};
    let index = -1;
    try {
        while (true) {
            index = charStream.index;
            const fieldName = exports.pToken(charStream);
            parserCombinators_1.pColon(charStream);
            pOWS(charStream);
            const value = pFieldValue(charStream);
            pOWS(charStream);
            pCRLF(charStream);
            result[fieldName] = value;
        }
    }
    catch (e) {
        if (parserCombinators_1.isParseError(e)) {
            charStream.index = index;
        }
        else {
            throw e;
        }
    }
    return result;
};
const parsePreProcessedHeaders = parserCombinators_1.parseWith(pHeaders);
exports.parseHeaders = (rawHeaders) => {
    var _b;
    const preProcessedHeaders = rawHeaders.replace(/\r?\n[\t ]+/g, " ");
    return (_b = parsePreProcessedHeaders(preProcessedHeaders)) !== null && _b !== void 0 ? _b : {};
};
