"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
const functions_1 = require("@reactive-js/core/lib/functions");
const option_1 = require("@reactive-js/core/lib/option");
const readonlyArray_1 = require("@reactive-js/core/lib/readonlyArray");
const HttpMessage_1 = require("./HttpMessage");
const cacheDirective_1 = require("./cacheDirective");
const httpContentInfo_1 = require("./httpContentInfo");
const httpHeaders_1 = require("./httpHeaders");
const httpPreferences_1 = require("./httpPreferences");
const httpRequestPreconditions_1 = require("./httpRequestPreconditions");
const httpResponse_1 = require("./httpResponse");
exports.createHttpRequest = (_a) => {
    var { body, cacheControl, contentInfo, expectContinue = false, headers = {}, httpVersionMajor = 1, httpVersionMinor = 1, method, preconditions, preferences, uri } = _a, rest = __rest(_a, ["body", "cacheControl", "contentInfo", "expectContinue", "headers", "httpVersionMajor", "httpVersionMinor", "method", "preconditions", "preferences", "uri"]);
    return (Object.assign(Object.assign({}, rest), { body, cacheControl: functions_1.pipe(cacheControl !== null && cacheControl !== void 0 ? cacheControl : [], readonlyArray_1.map(cc => (typeof cc === "string" ? cacheDirective_1.parseCacheDirectiveOrThrow(cc) : cc))), contentInfo: option_1.isSome(contentInfo)
            ? httpContentInfo_1.createHttpContentInfo(contentInfo)
            : httpContentInfo_1.parseHttpContentInfoFromHeaders(headers), expectContinue: expectContinue, headers: httpHeaders_1.filterHeaders(headers), httpVersionMajor: httpVersionMajor, httpVersionMinor: httpVersionMinor, method, preconditions: option_1.isSome(preconditions)
            ? httpRequestPreconditions_1.createHttpRequestPreconditions(preconditions)
            : httpRequestPreconditions_1.parseHttpRequestPreconditionsFromHeaders(headers), preferences: option_1.isSome(preferences)
            ? httpPreferences_1.createHttpPreferences(preferences)
            : httpPreferences_1.parseHttpPreferencesFromHeaders(headers), uri: typeof uri === "string" ? new URL(uri) : uri }));
};
exports.createRedirectHttpRequest = (request, response) => {
    const { contentInfo, method } = request;
    const { location, statusCode } = response;
    const redirectToGet = statusCode === 303 ||
        ((statusCode === 301 ||
            302 === 302) &&
            method === "POST");
    return Object.assign(Object.assign({}, request), { content: redirectToGet ? option_1.none : contentInfo, method: redirectToGet ? "GET" : method, uri: location });
};
const parseURIFromHeaders = (protocol, path, httpVersionMajor, headers) => {
    const forwardedProtocol = httpHeaders_1.getHeaderValue(headers, "X-Forwarded-Proto");
    const uriProtocol = option_1.isSome(forwardedProtocol)
        ? forwardedProtocol.split(/\s*,\s*/, 1)[0]
        : protocol;
    const forwardedHost = httpHeaders_1.getHeaderValue(headers, "X-Forwarded-Host");
    const http2Authority = headers[":authority"];
    const http1Host = httpHeaders_1.getHeaderValue(headers, "Host");
    const unfilteredHost = option_1.isSome(forwardedHost)
        ? forwardedHost
        : option_1.isSome(http2Authority) && httpVersionMajor >= 2
            ? http2Authority
            : option_1.isSome(http1Host)
                ? http1Host
                : "";
    const host = unfilteredHost.split(/\s*,\s*/, 1)[0];
    return new URL(`${uriProtocol}://${host}${path !== null && path !== void 0 ? path : ""}`);
};
exports.parseHttpRequestFromHeaders = ({ method, path, headers, httpVersionMajor, httpVersionMinor, isTransportSecure, body, }) => {
    const cacheControl = cacheDirective_1.parseCacheControlFromHeaders(headers);
    const contentInfo = httpContentInfo_1.parseHttpContentInfoFromHeaders(headers);
    const rawExpectHeader = httpHeaders_1.getHeaderValue(headers, "Expect");
    const expectContinue = rawExpectHeader === "100-continue";
    const preconditions = httpRequestPreconditions_1.parseHttpRequestPreconditionsFromHeaders(headers);
    const preferences = httpPreferences_1.parseHttpPreferencesFromHeaders(headers);
    const protocol = isTransportSecure ? "https" : "http";
    const uri = parseURIFromHeaders(protocol, path, httpVersionMajor, headers);
    return {
        body,
        cacheControl,
        contentInfo,
        expectContinue,
        headers,
        httpVersionMajor,
        httpVersionMinor,
        isTransportSecure,
        method,
        preconditions,
        preferences,
        uri,
    };
};
exports.writeHttpRequestHeaders = (request, writeHeader) => {
    const { expectContinue, preconditions } = request;
    if (expectContinue) {
        writeHeader("Expect", "100-continue");
    }
    if (option_1.isSome(preconditions)) {
        httpRequestPreconditions_1.writeHttpRequestPreconditionsHeaders(preconditions, writeHeader);
    }
    HttpMessage_1.writeHttpMessageHeaders(request, writeHeader);
};
exports.disallowProtocolAndHostForwarding = () => request => {
    const { httpVersionMajor, headers: oldHeaders, isTransportSecure, uri: oldUri, } = request;
    const { "x-forwarded-proto": xForwardedProto, "x-forwarded-host": xForwardedHost } = oldHeaders, headers = __rest(oldHeaders, ["x-forwarded-proto", "x-forwarded-host"]);
    const protocol = isTransportSecure ? "https" : "http";
    if (option_1.isNone(xForwardedProto) && option_1.isNone(xForwardedHost)) {
        return request;
    }
    else {
        const path = oldUri.pathname;
        const uri = parseURIFromHeaders(protocol, path, httpVersionMajor, headers);
        return Object.assign(Object.assign({}, request), { uri,
            headers });
    }
};
exports.httpRequestToUntypedHeaders = (request) => {
    const headers = {};
    exports.writeHttpRequestHeaders(request, (header, value) => (headers[header] = value));
    return headers;
};
exports.encodeHttpRequestWithUtf8 = HttpMessage_1.encodeHttpMessageWithUtf8;
exports.decodeHttpRequestWithCharset = HttpMessage_1.decodeHttpMessageWithCharset;
exports.toIOSourceHttpRequest = (req) => HttpMessage_1.toIOSourceHttpMessage(req);
exports.decodeHttpRequestContent = (decoderProvider) => req => {
    const { body, contentInfo } = req, rest = __rest(req, ["body", "contentInfo"]);
    if (option_1.isSome(contentInfo) && contentInfo.contentEncodings.length > 0) {
        const newBody = functions_1.pipe(contentInfo.contentEncodings, readonlyArray_1.map(encoding => {
            const decoder = decoderProvider[encoding];
            if (option_1.isNone(decoder)) {
                throw httpResponse_1.createHttpResponse({
                    statusCode: 415,
                    body: option_1.none,
                });
            }
            return decoder;
        }), readonlyArray_1.reduceRight((acc, decoder) => decoder(acc), functions_1.returns(body)));
        return Object.assign(Object.assign({}, rest), { contentInfo: {
                contentType: contentInfo.contentType,
                contentEncodings: [],
                contentLength: -1,
            }, body: newBody });
    }
    else {
        return req;
    }
};
exports.encodeHttpClientRequestContent = (encoderProvider, db = {}) => {
    const supportedEncodings = Object.keys(encoderProvider);
    const httpRequestIsCompressible = ({ contentInfo, }) => option_1.isSome(contentInfo) && httpContentInfo_1.contentIsCompressible(contentInfo, db);
    return request => {
        var _a;
        const { body, contentInfo } = request;
        if (option_1.isNone(contentInfo)) {
            return request;
        }
        const contentEncoding = ((_a = request === null || request === void 0 ? void 0 : request.acceptedEncodings) !== null && _a !== void 0 ? _a : []).find(encoding => supportedEncodings.includes(encoding));
        if (option_1.isNone(contentEncoding)) {
            return request;
        }
        const encode = option_1.isSome(contentEncoding) && httpRequestIsCompressible(request)
            ? encoderProvider[contentEncoding]
            : option_1.none;
        if (option_1.isNone(encode)) {
            return request;
        }
        return Object.assign(Object.assign({}, request), { body: encode(body), contentInfo: {
                contentType: contentInfo.contentType,
                contentEncodings: [contentEncoding],
                contentLength: -1,
            } });
    };
};
