"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
const functions_1 = require("@reactive-js/core/lib/functions");
const io_1 = require("@reactive-js/core/lib/io");
const option_1 = require("@reactive-js/core/lib/option");
const readonlyArray_1 = require("@reactive-js/core/lib/readonlyArray");
const HttpMessage_1 = require("./HttpMessage");
const cacheDirective_1 = require("./cacheDirective");
const entityTag_1 = require("./entityTag");
const httpContentInfo_1 = require("./httpContentInfo");
const httpDateTime_1 = require("./httpDateTime");
const httpHeaders_1 = require("./httpHeaders");
const httpPreferences_1 = require("./httpPreferences");
exports.createHttpResponse = (_a) => {
    var { body, cacheControl, contentInfo, etag, expires, headers = {}, lastModified, location, preferences, statusCode, vary } = _a, rest = __rest(_a, ["body", "cacheControl", "contentInfo", "etag", "expires", "headers", "lastModified", "location", "preferences", "statusCode", "vary"]);
    return (Object.assign(Object.assign({}, rest), { body, cacheControl: functions_1.pipe(cacheControl !== null && cacheControl !== void 0 ? cacheControl : [], readonlyArray_1.map(cc => (typeof cc === "string" ? cacheDirective_1.parseCacheDirectiveOrThrow(cc) : cc))), contentInfo: option_1.isSome(contentInfo)
            ? httpContentInfo_1.createHttpContentInfo(contentInfo)
            : httpContentInfo_1.parseHttpContentInfoFromHeaders(headers), etag: typeof etag === "string" ? entityTag_1.parseETagOrThrow(etag) : etag, expires, headers: httpHeaders_1.filterHeaders(headers !== null && headers !== void 0 ? headers : {}), lastModified: typeof lastModified === "string"
            ? httpDateTime_1.parseHttpDateTime(lastModified)
            : lastModified instanceof Date
                ? lastModified.getTime()
                : lastModified, location: typeof location === "string" ? new URL(location) : location, preferences: option_1.isSome(preferences)
            ? httpPreferences_1.createHttpPreferences(preferences)
            : httpPreferences_1.parseHttpPreferencesFromHeaders(headers), statusCode, vary: vary !== null && vary !== void 0 ? vary : [] }));
};
exports.parseHttpResponseFromHeaders = (statusCode, headers, body) => {
    var _a, _b, _c;
    const cacheControl = cacheDirective_1.parseCacheControlFromHeaders(headers);
    const contentInfo = httpContentInfo_1.parseHttpContentInfoFromHeaders(headers);
    const etag = entityTag_1.parseETag((_a = httpHeaders_1.getHeaderValue(headers, "ETag")) !== null && _a !== void 0 ? _a : "");
    const expires = httpDateTime_1.parseHttpDateTime((_b = httpHeaders_1.getHeaderValue(headers, "Expires")) !== null && _b !== void 0 ? _b : "");
    const lastModified = httpDateTime_1.parseHttpDateTime((_c = httpHeaders_1.getHeaderValue(headers, "Last-Modified")) !== null && _c !== void 0 ? _c : "");
    const locationHeader = headers.location;
    const location = option_1.isSome(locationHeader) ? new URL(locationHeader) : option_1.none;
    const preferences = httpPreferences_1.parseHttpPreferencesFromHeaders(headers);
    const vary = [];
    return {
        body,
        cacheControl,
        contentInfo,
        etag,
        expires,
        lastModified,
        headers,
        location,
        preferences,
        statusCode,
        vary,
    };
};
exports.writeHttpResponseHeaders = (response, writeHeader) => {
    const { etag, expires, lastModified, location, vary } = response;
    if (option_1.isSome(etag)) {
        writeHeader("ETag", entityTag_1.entityTagToString(etag));
    }
    if (option_1.isSome(expires)) {
        writeHeader("Expires", httpDateTime_1.httpDateTimeToString(expires));
    }
    if (option_1.isSome(lastModified)) {
        writeHeader("Last-Modified", httpDateTime_1.httpDateTimeToString(lastModified));
    }
    if (option_1.isSome(location)) {
        writeHeader("Location", location.toString());
    }
    if (vary.length > 0) {
        writeHeader("Vary", functions_1.pipe(vary, readonlyArray_1.join(",")));
    }
    HttpMessage_1.writeHttpMessageHeaders(response, writeHeader);
};
exports.checkIfNotModified = ({ cacheControl, method, preconditions, }) => response => {
    var _a, _b;
    const { etag, lastModified } = response;
    const { statusCode, contentInfo: _ } = response, responseWithoutContent = __rest(response, ["statusCode", "contentInfo"]);
    const methodSupportsConditionalResponse = method === "GET" || method === "HEAD";
    const statusCodeSupportsConditionalResponse = statusCode >= 200 && statusCode < 300;
    const isNoCacheRequest = cacheControl.findIndex(({ directive }) => directive === "no-cache") >= 0;
    const etagMatch = option_1.isSome(etag) &&
        ((preconditions === null || preconditions === void 0 ? void 0 : preconditions.ifNoneMatch) === "*" ||
            ((_a = preconditions === null || preconditions === void 0 ? void 0 : preconditions.ifNoneMatch) !== null && _a !== void 0 ? _a : []).findIndex(({ tag }) => tag === etag.tag) >= 0);
    const notModifiedSince = (lastModified !== null && lastModified !== void 0 ? lastModified : Number.MAX_SAFE_INTEGER) <=
        ((_b = preconditions === null || preconditions === void 0 ? void 0 : preconditions.ifModifiedSince) !== null && _b !== void 0 ? _b : Number.MIN_SAFE_INTEGER);
    const match = option_1.isSome(etag) &&
        option_1.isSome(preconditions === null || preconditions === void 0 ? void 0 : preconditions.ifNoneMatch) &&
        option_1.isSome(lastModified) &&
        option_1.isSome(preconditions === null || preconditions === void 0 ? void 0 : preconditions.ifModifiedSince)
        ? notModifiedSince && etagMatch
        : option_1.isSome(etag) && option_1.isSome(preconditions === null || preconditions === void 0 ? void 0 : preconditions.ifNoneMatch)
            ? etagMatch
            : notModifiedSince;
    return methodSupportsConditionalResponse &&
        statusCodeSupportsConditionalResponse &&
        !isNoCacheRequest &&
        match
        ? Object.assign(Object.assign({}, responseWithoutContent), { statusCode: 304 }) : response;
};
exports.encodeHttpResponseWithUtf8 = HttpMessage_1.encodeHttpMessageWithUtf8;
exports.decodeHttpResponseWithCharset = HttpMessage_1.decodeHttpMessageWithCharset;
exports.toIOSourceHttpResponse = (resp) => HttpMessage_1.toIOSourceHttpMessage(resp);
exports.decodeHttpResponseContent = (decoderProvider) => resp => {
    const { body, contentInfo } = resp, rest = __rest(resp, ["body", "contentInfo"]);
    if (option_1.isSome(contentInfo) && contentInfo.contentEncodings.length > 0) {
        const decoders = functions_1.pipe(contentInfo.contentEncodings, readonlyArray_1.map(encoding => decoderProvider[encoding]));
        const supportsDecodings = functions_1.pipe(decoders, readonlyArray_1.everySatisfy(option_1.isSome));
        if (supportsDecodings) {
            return Object.assign(Object.assign({}, rest), { contentInfo: {
                    contentType: contentInfo.contentType,
                    contentEncodings: [],
                    contentLength: -1,
                }, body: functions_1.pipe(decoders, readonlyArray_1.reduceRight(functions_1.updaterReducer, functions_1.returns(body))) });
        }
        else {
            return exports.createHttpResponse({
                statusCode: 415,
                body: io_1.empty(),
            });
        }
    }
    else {
        return resp;
    }
};
exports.encodeHttpResponseContent = (encoderProvider, db = {}) => {
    const supportedEncodings = Object.keys(encoderProvider);
    const httpResponseIsCompressible = (response) => {
        const noTransformResponse = response.cacheControl.findIndex(({ directive }) => directive === "no-transform") >= 0;
        const { contentInfo } = response;
        return (!noTransformResponse &&
            option_1.isSome(contentInfo) &&
            httpContentInfo_1.contentIsCompressible(contentInfo, db));
    };
    return (request) => response => {
        const { body, contentInfo, vary } = response;
        if (option_1.isNone(contentInfo)) {
            return response;
        }
        const { preferences } = request;
        const shouldEncode = httpResponseIsCompressible(response);
        const acceptedEncodings = shouldEncode && option_1.isSome(preferences) ? preferences.acceptedEncodings : [];
        const contentEncoding = acceptedEncodings.find(encoding => supportedEncodings.includes(encoding));
        if (option_1.isNone(contentEncoding)) {
            return response;
        }
        const encode = option_1.isSome(contentEncoding)
            ? encoderProvider[contentEncoding]
            : option_1.none;
        if (option_1.isNone(encode)) {
            return response;
        }
        return Object.assign(Object.assign({}, response), { body: encode(body), contentInfo: {
                contentType: contentInfo.contentType,
                contentEncodings: [contentEncoding],
                contentLength: -1,
            }, vary: [...vary, "Accept-Encoding"] });
    };
};
exports.createHttpErrorResponse = (e) => {
    const statusCode = e instanceof URIError
        ? 400
        : 500;
    return exports.createHttpResponse({
        statusCode,
        body: e,
    });
};
