"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const functions_1 = require("@reactive-js/core/lib/functions");
const option_1 = require("@reactive-js/core/lib/option");
const readonlyArray_1 = require("@reactive-js/core/lib/readonlyArray");
const createSegments = (path) => {
    const root = { name: "" };
    let acc = root;
    for (const name of path.split("/")) {
        const child = { name };
        acc.child = child;
        acc = child;
    }
    return root;
};
const serializeSegments = (segment) => {
    let result = segment.name;
    while (option_1.isSome(segment.child)) {
        segment = segment.child;
        result += `/${segment.name}`;
    }
    return result;
};
const emptyRouter = {
    name: "",
    children: {},
};
const addHandler = (router, { name, child }, handler) => {
    var _a;
    if (option_1.isNone(child)) {
        return {
            name,
            handler,
            children: router.children,
        };
    }
    else {
        const childName = child.name.startsWith(":") ? ":" : child.name;
        const childRouter = (_a = router.children[childName]) !== null && _a !== void 0 ? _a : emptyRouter;
        const newChildRouter = addHandler(childRouter, child, handler);
        return {
            name,
            handler: router.handler,
            children: Object.assign(Object.assign({}, router.children), { [childName]: newChildRouter }),
        };
    }
};
const findHandler = (router, segment, params) => {
    const { child } = segment;
    const { handler } = router;
    if (option_1.isNone(child) && option_1.isSome(handler)) {
        return [handler, params];
    }
    if (option_1.isNone(child)) {
        return option_1.none;
    }
    const nameRouter = router.children[child.name];
    const nameRouterResult = option_1.isSome(nameRouter)
        ? findHandler(nameRouter, child, params)
        : option_1.none;
    if (option_1.isSome(nameRouterResult)) {
        return nameRouterResult;
    }
    const paramRouter = router.children[":"];
    const paramRouterResult = option_1.isSome(paramRouter)
        ? findHandler(paramRouter, child, Object.assign(Object.assign({}, params), { [paramRouter.name]: child.name }))
        : option_1.none;
    if (option_1.isSome(paramRouterResult)) {
        return paramRouterResult;
    }
    const globRouter = router.children["*"];
    const globRouterHandler = globRouter === null || globRouter === void 0 ? void 0 : globRouter.handler;
    if (option_1.isSome(globRouterHandler)) {
        const newParams = Object.assign(Object.assign({}, params), { ["*"]: serializeSegments(child) });
        return [globRouterHandler, newParams];
    }
    return option_1.none;
};
exports.createRoutingHttpServer = (routes, notFoundHandler) => {
    const router = functions_1.pipe(routes, readonlyArray_1.fromObject(), readonlyArray_1.reduce((acc, [path, handler]) => addHandler(acc, createSegments(path), handler), functions_1.returns(emptyRouter)));
    return (request) => {
        const segments = createSegments(request.uri.pathname);
        const result = findHandler(router, segments, {});
        if (option_1.isSome(result)) {
            const [handler, params] = result;
            const requestWithParams = Object.assign(Object.assign({}, request), { params });
            return handler(requestWithParams);
        }
        else {
            return notFoundHandler(request);
        }
    };
};
