"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const functions_1 = require("@reactive-js/core/lib/functions");
const parserCombinators_1 = require("@reactive-js/core/lib/internal/parserCombinators");
const readonlyArray_1 = require("@reactive-js/core/lib/readonlyArray");
const httpGrammar_1 = require("./httpGrammar");
exports.pMediaType = charStream => {
    const type = httpGrammar_1.pToken(charStream);
    parserCombinators_1.pForwardSlash(charStream);
    const subtype = httpGrammar_1.pToken(charStream);
    const params = httpGrammar_1.pParams(charStream);
    return {
        type,
        subtype,
        params,
    };
};
exports.parseMediaType = parserCombinators_1.parseWith(exports.pMediaType);
exports.parseMediaTypeOrThrow = parserCombinators_1.parseWithOrThrow(exports.pMediaType);
exports.mediaTypeToString = ({ type, subtype, params, }) => functions_1.pipe(params, readonlyArray_1.fromObject(), readonlyArray_1.map(([k, v]) => `${k}=${httpGrammar_1.toTokenOrQuotedString(v)}`), readonlyArray_1.join("; "), stringParams => `${type}/${subtype}${stringParams.length > 0 ? ";" + stringParams : ""}`);
const compressionBlacklist = [
    "text/event-stream",
];
const textSubtypes = ["html", "json", "text", "xml"];
exports.mediaTypeIsCompressible = ({ type, subtype }, db) => {
    var _a, _b;
    const mediaType = exports.mediaTypeToString({ type, subtype, params: {} });
    const blackListed = compressionBlacklist.includes(mediaType);
    const compressible = (_b = (_a = db[mediaType]) === null || _a === void 0 ? void 0 : _a.compressible) !== null && _b !== void 0 ? _b : false;
    const typeIsText = type === "text";
    const subtypeIsText = functions_1.pipe(textSubtypes, readonlyArray_1.keep(x => subtype.endsWith(x)), readonlyArray_1.length) > 0;
    return !blackListed && (compressible || typeIsText || subtypeIsText);
};
