"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const http_1 = require("http");
const https_1 = require("https");
const url_1 = require("url");
const disposable_1 = require("@reactive-js/core/lib/disposable");
const functions_1 = require("@reactive-js/core/lib/functions");
const node_1 = require("@reactive-js/core/lib/node");
const observable_1 = require("@reactive-js/core/lib/observable");
const option_1 = require("@reactive-js/core/lib/option");
const streamable_1 = require("@reactive-js/core/lib/streamable");
const http_2 = require("../../http");
class ResponseBody extends disposable_1.AbstractDisposable {
    constructor(resp) {
        super();
        this.resp = resp;
        this.consumed = false;
        disposable_1.addTeardown(this, _ => {
            resp.removeAllListeners();
            resp.destroy();
        });
        const onError = disposable_1.toErrorHandler(this);
        resp.on("error", onError);
    }
    stream(scheduler, replayCount) {
        if (this.consumed) {
            throw new Error("Response body already consumed");
        }
        this.consumed = true;
        const responseStream = streamable_1.stream(node_1.createReadableIOSource(functions_1.defer(this.resp, node_1.createDisposableNodeStream)), scheduler, replayCount);
        disposable_1.bindDisposables(this, responseStream);
        return responseStream;
    }
}
exports.createHttpClient = (options = {}) => request => observable_1.using((scheduler) => {
    const { method, uri } = request;
    const url = uri instanceof url_1.URL ? uri : new url_1.URL(uri.toString());
    const headers = http_2.httpRequestToUntypedHeaders(request);
    const nodeRequestOptions = Object.assign(Object.assign({}, options), { headers,
        method });
    const req = uri.protocol === "https:"
        ? https_1.request(url, nodeRequestOptions)
        : uri.protocol === "http:"
            ? http_1.request(url, nodeRequestOptions)
            : (() => {
                throw new Error();
            })();
    const requestSink = streamable_1.stream(node_1.createWritableIOSink(functions_1.defer(req, node_1.createDisposableNodeStream)), scheduler);
    const requestBody = streamable_1.stream(request.body, scheduler);
    const onContinue = () => {
        const reqSubscription = functions_1.pipe(requestSink, observable_1.onNotify(observable_1.dispatchTo(requestBody)), observable_1.subscribe(scheduler));
        const dataSubscription = functions_1.pipe(requestBody, observable_1.onNotify(observable_1.dispatchTo(requestSink)), observable_1.subscribe(scheduler));
        disposable_1.addDisposable(dataSubscription, reqSubscription),
            disposable_1.addDisposable(requestBody, dataSubscription);
        disposable_1.addDisposable(requestBody, requestSink);
    };
    if (request.expectContinue) {
        req.on("continue", onContinue);
    }
    else {
        onContinue();
    }
    const responseSubject = observable_1.createSubject();
    const onResponse = (resp) => {
        var _a;
        disposable_1.dispose(requestBody);
        const body = new ResponseBody(resp);
        disposable_1.bindDisposables(responseSubject, body);
        const response = http_2.parseHttpResponseFromHeaders((_a = resp.statusCode) !== null && _a !== void 0 ? _a : -1, resp.headers, body);
        observable_1.dispatch(responseSubject, response);
    };
    req.on("response", onResponse);
    return [requestBody, responseSubject];
}, (requestBody, responseSubject) => observable_1.merge(functions_1.pipe(requestBody, observable_1.scan(([increment, count], ev) => ev.type === 1
    ? [ev.data.length, count + increment]
    : [-1, count + increment], functions_1.returns([0, 0])), observable_1.map(([increment, count]) => increment < 0
    ? { type: 3 }
    : count > 0
        ? {
            type: 2,
            count,
        }
        : option_1.none), observable_1.keepType(option_1.isSome)), functions_1.pipe(responseSubject, observable_1.map(response => ({
    type: 4,
    response,
})))));
