"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const disposable_1 = require("@reactive-js/core/lib/disposable");
const functions_1 = require("@reactive-js/core/lib/functions");
const node_1 = require("@reactive-js/core/lib/node");
const observable_1 = require("@reactive-js/core/lib/observable");
const streamable_1 = require("@reactive-js/core/lib/streamable");
const http_1 = require("../../http");
const writeResponseMessage = (serverResponse) => (response) => {
    serverResponse.statusCode = response.statusCode;
    http_1.writeHttpResponseHeaders(response, (header, value) => serverResponse.setHeader(header, value));
    serverResponse.flushHeaders();
};
const writeResponseBody = (responseBody) => ({ body, }) => streamable_1.sink(body, responseBody);
const defaultOnError = (e) => {
    console.log(e);
};
exports.createHttpRequestListener = (handler, scheduler, options = {}) => {
    const { onError = defaultOnError } = options;
    const handleRequest = (request, response) => {
        var _a;
        const { method, url: path = "/", headers, httpVersionMajor, httpVersionMinor, } = request.value;
        const isTransportSecure = (_a = request.value.socket.encrypted) !== null && _a !== void 0 ? _a : false;
        const requestBody = node_1.createReadableIOSource(functions_1.returns(request));
        const responseBody = node_1.createWritableIOSink(functions_1.returns(response));
        return functions_1.pipe(functions_1.defer({
            method: method,
            path,
            headers: headers,
            httpVersionMajor,
            httpVersionMinor,
            isTransportSecure,
            body: requestBody,
        }, http_1.parseHttpRequestFromHeaders), observable_1.compute(), observable_1.await_(handler), observable_1.onNotify(writeResponseMessage(response.value)), observable_1.await_(writeResponseBody(responseBody)), observable_1.catchError(onError));
    };
    return (req, resp) => {
        const request = node_1.createDisposableNodeStream(req);
        const response = node_1.createDisposableNodeStream(resp);
        const handlerSubscription = functions_1.pipe(handleRequest(request, response), observable_1.subscribe(scheduler));
        disposable_1.addDisposable(handlerSubscription, request);
        disposable_1.addDisposable(response, handlerSubscription);
    };
};
