import { addDisposable, } from "@reactive-js/core/lib/disposable.mjs";
import { defer, pipe, returns, } from "@reactive-js/core/lib/functions.mjs";
import { createReadableIOSource, createWritableIOSink, createDisposableNodeStream, } from "@reactive-js/core/lib/node.mjs";
import { await_, catchError, onNotify, subscribe, compute, } from "@reactive-js/core/lib/observable.mjs";
import { sink } from "@reactive-js/core/lib/streamable.mjs";
import { writeHttpResponseHeaders, parseHttpRequestFromHeaders, } from "../../http.mjs";
const writeResponseMessage = (serverResponse) => (response) => {
    serverResponse.statusCode = response.statusCode;
    writeHttpResponseHeaders(response, (header, value) => serverResponse.setHeader(header, value));
    serverResponse.flushHeaders();
};
const writeResponseBody = (responseBody) => ({ body, }) => sink(body, responseBody);
const defaultOnError = (e) => {
    console.log(e);
};
export const createHttpRequestListener = (handler, scheduler, options = {}) => {
    const { onError = defaultOnError } = options;
    const handleRequest = (request, response) => {
        var _a;
        const { method, url: path = "/", headers, httpVersionMajor, httpVersionMinor, } = request.value;
        const isTransportSecure = (_a = request.value.socket.encrypted) !== null && _a !== void 0 ? _a : false;
        const requestBody = createReadableIOSource(returns(request));
        const responseBody = createWritableIOSink(returns(response));
        return pipe(defer({
            method: method,
            path,
            headers: headers,
            httpVersionMajor,
            httpVersionMinor,
            isTransportSecure,
            body: requestBody,
        }, parseHttpRequestFromHeaders), compute(), await_(handler), onNotify(writeResponseMessage(response.value)), await_(writeResponseBody(responseBody)), catchError(onError));
    };
    return (req, resp) => {
        const request = createDisposableNodeStream(req);
        const response = createDisposableNodeStream(resp);
        const handlerSubscription = pipe(handleRequest(request, response), subscribe(scheduler));
        addDisposable(handlerSubscription, request);
        addDisposable(response, handlerSubscription);
    };
};
