import { pipe } from "../../../../../core/mod/lib/functions.ts";
import {
  pForwardSlash,
  Parser,
  parseWith,
  parseWithOrThrow,
} from "../../../../../core/mod/lib/internal/parserCombinators.ts";
import {
  fromObject,
  map,
  join,
  keep,
  length,
} from "../../../../../core/mod/lib/readonlyArray.ts";
import { pParams, pToken, toTokenOrQuotedString } from "./httpGrammar.ts";
import { MediaType } from "./interfaces.ts";

export const pMediaType: Parser<MediaType> = charStream => {
  const type = pToken(charStream);
  pForwardSlash(charStream);
  const subtype = pToken(charStream);
  const params = pParams(charStream);

  return {
    type,
    subtype,
    params,
  };
};

export const parseMediaType = parseWith(pMediaType);
export const parseMediaTypeOrThrow = parseWithOrThrow(pMediaType);

export const mediaTypeToString = ({
  type,
  subtype,
  params,
}: MediaType): string =>
  pipe(
    params,
    fromObject(),
    map(([k, v]) => `${k}=${toTokenOrQuotedString(v)}`),
    join("; "),
    stringParams =>
      `${type}/${subtype}${stringParams.length > 0 ? ";" + stringParams : ""}`,
  );

const compressionBlacklist = [
  "text/event-stream", // Browser's don't seem to support compressed event streams
];

const textSubtypes = ["html", "json", "text", "xml"];

export const mediaTypeIsCompressible = (
  { type, subtype }: MediaType,
  db: {
    [key: string]: {
      compressible?: boolean;
    };
  },
) => {
  const mediaType = mediaTypeToString({ type, subtype, params: {} });
  const blackListed = compressionBlacklist.includes(mediaType);
  const compressible = db[mediaType]?.compressible ?? false;
  const typeIsText = type === "text";
  const subtypeIsText =
    pipe(
      textSubtypes,
      keep(x => subtype.endsWith(x)),
      length,
    ) > 0;

  return !blackListed && (compressible || typeIsText || subtypeIsText);
};
